package sun.reflect.generics.scope;

import java.lang.reflect.TypeVariable;
import java.lang.reflect.GenericDeclaration;

public abstract class AbstractScope<D extends GenericDeclaration> implements Scope
{
    private final D recvr;
    private volatile Scope enclosingScope;
    
    protected AbstractScope(D recvr) {
        this.recvr = recvr;
    }
    
    protected D getRecvr() {
        return this.recvr;
    }
    
    protected abstract Scope computeEnclosingScope();
    
    protected Scope getEnclosingScope() {
        Scope enclosingScope = this.enclosingScope;
        if (enclosingScope == null) {
            enclosingScope = this.computeEnclosingScope();
            this.enclosingScope = enclosingScope;
            return enclosingScope;
        }
        return enclosingScope;
    }
    
    @Override
    public TypeVariable<?> lookup(String anObject) {
        TypeVariable[] typeParameters = this.getRecvr().getTypeParameters();
        int length = typeParameters.length;
        int i = 0;
        while (i < length) {
            TypeVariable<Object> typeVariable = typeParameters[i];
            if (typeVariable.getName().equals(anObject)) {
                return typeVariable;
            }
            ++i;
        }
        return this.getEnclosingScope().lookup(anObject);
    }
}
